package Usbnotify;

/**********************************************************************
* $Id: $
*
* written by Timo Boettcher
* usbnotify (near) spida (.) net
*
* (C) 2008,2009 Timo Boettcher
* This is free software. You may redistribute copies of it under the
* terms of the GNU General Public License (Version 2)
* <http://www.gnu.org/licenses/gpl.html>.
* There is NO WARRANTY, to the extent permitted by law.
*
**********************************************************************/

import ch.ntb.usb.*;

public class Usbnotify {
	public static final short USB_ID_VENDOR			= 0x16c0;
	public static final short USB_ID_PRODUCT		= 0x05dc;
	public static final String USB_MANUFACTURER		= "Spida.net";
	public static final String USB_PRODUCT			= "Usbnotify";

	public static final short FUNC_VERSION			= 0;
	public static final short FUNC_RESET			= 1;
	public static final short FUNC_ECHO			= 2;
	public static final short FUNC_ENUMERATE		= 3;
	public static final short FUNC_GET_STATUS		= 4;
	public static final short FUNC_PREPARE_COLOR_PARAM	= 5;
	public static final short FUNC_PREPARE_FADE_PARAM	= 6;
	public static final short FUNC_EXEC_SET_COLOR		= 7;
	public static final short FUNC_EXEC_FADETO_COLOR	= 8;

	public static final int requesttype = ch.ntb.usb.USB.REQ_TYPE_TYPE_VENDOR | ch.ntb.usb.USB.REQ_TYPE_RECIP_INTERFACE | ch.ntb.usb.USB.REQ_TYPE_DIR_DEVICE_TO_HOST;

	private short usbVendorId = USB_ID_VENDOR;
	private short usbProductId = USB_ID_PRODUCT;

	private long devicehandle = 0;

	private Device device = null;
	private Usb_Device usbdevice = null;

	Usbnotify() {
		LibusbJava.usb_init();
		LibusbJava.usb_find_busses();
		LibusbJava.usb_find_devices();
		System.out.println("searching...");
		String devicefilename = this.FindDevice(this.usbVendorId, this.usbProductId);

		System.out.println("opening...");
		this.OpenDevice(devicefilename);

		System.out.println("resetting...");
		this.Reset(0, 0);

		System.out.println("closing...");
		this.CloseDevice(devicefilename);
	}


	private String FindDevice(int usbVendorId, int usbProductId) {
		String devicefilename;
		for (Usb_Bus bus = LibusbJava.usb_get_busses(); bus != null; bus = bus.getNext()) {
			for (Usb_Device dev = bus.getDevices(); dev != null; dev = dev.getNext()) {
				Usb_Device_Descriptor devDesc = dev.getDescriptor();

				devicehandle = LibusbJava.usb_open(dev);
				if ((devDesc.getIdVendor() == this.usbVendorId) && (devDesc.getIdProduct() == this.usbProductId)) {
					System.out.println(Integer.toHexString(devDesc.getIdVendor()) + " " + Integer.toHexString(devDesc.getIdProduct()));
					System.out.println(LibusbJava.usb_get_string_simple(devicehandle, (int)devDesc.getIManufacturer()) + " " + LibusbJava.usb_get_string_simple(devicehandle, (int)devDesc.getIProduct()) + " " + LibusbJava.usb_get_string_simple(devicehandle, (int)devDesc.getISerialNumber()));

					if ((LibusbJava.usb_get_string_simple(devicehandle, (int)devDesc.getIManufacturer()).equals(this.USB_MANUFACTURER)) && (LibusbJava.usb_get_string_simple(devicehandle, (int)devDesc.getIProduct()).equals(this.USB_PRODUCT))) {

						devicefilename = dev.getFilename();
						usbdevice = dev;
						System.out.println("FOUND");

					}
				}
			}
		}
		return devicefilename;
	}

	private void OpenDevice(String devicefilename) {
		this.device = USB.getDevice(usbVendorId, usbProductId, devicefilename);
		this.devicehandle = LibusbJava.usb_open(usbdevice);
	}

	private void CloseDevice(String devicefilename) {
		LibusbJava.usb_close(this.devicehandle);
		this.device = null;
		this.devicehandle = 0;
	}

	public void Version(int busid, int deviceid) {
		byte[] buffer = {(byte) 0, (byte) 0, (byte) 0, (byte) 0 , (byte) 0, (byte) 0};
		int result = LibusbJava.usb_control_msg(this.devicehandle, requesttype, FUNC_VERSION, 0, 0, buffer, buffer.length, 1000);
		System.out.format("Usbnotify at 0 has version (%d, %d, %d, %d, %d, %d)\n", buffer[0], buffer[1], buffer[2], buffer[3], buffer[4], buffer[5]);
	}

	public void Reset(int busid, int deviceid) {
		System.out.println("Resetting device at 0");
		byte[] buffer = {(byte) 0, (byte) 0};
		int result = LibusbJava.usb_control_msg(this.devicehandle, requesttype, FUNC_RESET, 0, 0, buffer, buffer.length, 1000);
	}

	public void Enumerate() {
	}

	public void EchoDevice(boolean longtest) {
	}

	public void Status(int busid, int deviceid) {
		byte[] buffer = {(byte) 0, (byte) 0 , (byte) 0, (byte) 0};
		int result = LibusbJava.usb_control_msg(this.devicehandle, requesttype, FUNC_GET_STATUS, 0, 0, buffer, buffer.length, 1000);
		System.out.format("(%d, %d, %d, %d)\n" , buffer[0]&0xff, buffer[1]&0xff, buffer[2]&0xff, buffer[3]&0xff);
	}

	public void SetColor(int busid, int deviceid, int R, int G, int B, int W) {
		int RG = (R<<8)+G;
		int BW = (B<<8)+W;

		byte[] buffer = {(byte) 0, (byte) 0};
		int result = LibusbJava.usb_control_msg(this.devicehandle, requesttype, FUNC_PREPARE_COLOR_PARAM, RG, BW, buffer, buffer.length, 1000);
		result = LibusbJava.usb_control_msg(this.devicehandle, requesttype, FUNC_EXEC_SET_COLOR, 0, 0, buffer, buffer.length, 1000);
	}

	public void FadeColor(int busid, int deviceid, int R, int G, int B, int W, int time) {
		int RG = (R<<8)+G;
		int BW = (B<<8)+W;

		byte[] buffer = {(byte) 0, (byte) 0};
		int result = LibusbJava.usb_control_msg(this.devicehandle, requesttype, FUNC_PREPARE_COLOR_PARAM, RG, BW, buffer, buffer.length, 1000);
		result = LibusbJava.usb_control_msg(this.devicehandle, requesttype, FUNC_PREPARE_FADE_PARAM, time, 0, buffer, buffer.length, 1000);
		result = LibusbJava.usb_control_msg(this.devicehandle, requesttype, FUNC_EXEC_FADETO_COLOR, 0, 0, buffer, buffer.length, 1000);
	}

}
